/*
  LiquidCrystal_I2Clibrary download address:http://osoyoo.com/driver/LiquidCrystal_I2C.zip
  注意：LiquidCrystal_I2C要使用上面下载的LiquidCrystal_I2C版本，其他版本的可能会出现如下bug：
  如果要在屏上打印String字符串，用print("String")方法只能显示String首字母S
*/
#include <LiquidCrystal_I2C.h>

#define YL69 A0          //define as YL69 the Pin A0 used to connect the Sensor
#define RELAY 6         //define as RELAY the Pin 4 used to connect the Sensor
#define CLK_PIN 2 // Encoder's clk pin connected to D2
#define DT_PIN 4   //Encoder's dt pin connected to D4
#define SW_PIN 3   //Encoder's sw pin connected to D3

volatile long count = 0;//编码器旋转时作为计数器
unsigned long t = 0;
volatile int inter_num=0;//按按键次数
int soilhum=60;//土壤湿度临界值
/*The (int soil=0) is the soil variable which turns the reading that comes 
 in on the sensor into a %, this will be an important variable in this project*/
int soil=0;//实时土壤湿度
/*run ic2_scanner sketch and get the IC2 address, which is 0x27 in my case,it could be 0x27 in many cases
 run ic2_scanner sketch:http://osoyoo.com/wp-content/uploads/samplecode/ic2_scanner.txt 
 */
LiquidCrystal_I2C lcd(0x27, 16, 2); 

//定时器初始化函数
void time1_init(void){
  cli();//关全局中断
  TCCR1A=0; //寄存器A是配置PWM的，这里我们只是使用定时功能，故把TCCR1A寄存器置零
  TCCR1B=(1<<CS12)| (0<<CS11)|(1<<CS10);//寄存器B是配置定时功能的，现在配置的是1024分频
  TCNT1=0xC2F6; //计数器初值，1s定时
  sei(); //开全局中断 
}

void setup() {
  time1_init();
  // 下降沿代表旋转编码器被转动
  attachInterrupt(digitalPinToInterrupt(CLK_PIN), rotaryEncoderChanged, FALLING);
  //按键按下表示进入设置模式
  attachInterrupt(digitalPinToInterrupt(SW_PIN), setmode, FALLING);
  pinMode(CLK_PIN, INPUT_PULLUP);// initialize the digital pin as Pull-up input. 
  pinMode(DT_PIN, INPUT_PULLUP); // initialize the digital pin as Pull-up input. 
  pinMode(SW_PIN, INPUT_PULLUP); // initialize the digital pin as Pull-up input. 

  pinMode(RELAY,OUTPUT);// initialize the digital pin as an output. 
  digitalWrite(RELAY,LOW);//set digital pin output low
  Serial.begin(9600);// initialize serial communication at 9600 bits per second
  lcd.begin();// initialize the LCD
  lcd.backlight();//enable LCD backlight
  
}

int get_YL69()//Get YL-69 sensor data
{
  int YL_Value = analogRead(YL69);// read the input on analog pin A0:
  YL_Value=constrain(YL_Value, 485, 1023);//Constrains YL_Value variable to be within a range. 
  soil = map(YL_Value, 485, 1023, 100, 0);//map the value to a percentage
  Serial.print("Soil Humidity:");
  // print out the soil water percentage you calculated:
  Serial.print(soil);// print out the value you read:
  Serial.println("%");
  return soil;// return the soil water percentage you calculated:
}

void relay_on()//turn the Relay on 
{
    digitalWrite(RELAY,HIGH);//HIGH is the voltage level
}
void relay_off()//turn the Relay off 
{
    digitalWrite(RELAY,LOW);//LOW is the voltage level
}

void homepage(void){
  int soil=get_YL69();
   lcd.clear();
   lcd.setCursor(0, 0);//set the cursor to column 0, line 0
   lcd.print("soil:");
  /*显示单位*/
   lcd.setCursor(8, 0);//set the cursor to column 8, line 0 
   lcd.print("%");
  /*显示数据*/
   lcd.setCursor(5, 0);//set the cursor to column 5, line 0 
   lcd.print(soil);
   delay(1000);
}

void page_1(void){
    lcd.clear();
    lcd.setCursor(0, 0);//set the cursor to column 0, line 0 
    lcd.print("You will change ");
    lcd.setCursor(0, 1);//set the cursor to column 0, line 1 
    lcd.print("soil humidity");
    delay(1000);
}

void page_2(void){
  lcd.clear();
  lcd.setCursor(0, 0);//set the cursor to column 0, line 0 
  lcd.print("soilhum:");
  lcd.setCursor(0, 1);//set the cursor to column 0, line 1 
  lcd.print(count);
  lcd.setCursor(3, 1);//set the cursor to column 3, line 1 
  lcd.print("%");
  //soilhum=count;
  Serial.print("soilhum=");
  Serial.println(count);
  delay(1000);
}
// the loop routine runs over and over again forever:
void loop() {
  detachInterrupt(digitalPinToInterrupt(CLK_PIN));//disable interrupt 0
  
  if (inter_num==0){
    homepage();
     if (soil < soilhum) {
         TIMSK1=(1<<TOIE1); //溢出中断使能
      }
     else relay_off();
  }
      
   //进入设置模式
  else if(inter_num==1){
    page_1();
    relay_off();
    TIMSK1=(0<<TOIE1);
  }
      
   else if(inter_num==2){
       attachInterrupt(digitalPinToInterrupt(CLK_PIN), rotaryEncoderChanged, FALLING);//enable interrupt 0
       page_2();
       relay_off();
       TIMSK1=(0<<TOIE1);
   }
   else if(inter_num==3){
       detachInterrupt(digitalPinToInterrupt(CLK_PIN));//disable interrupt 0
       soilhum=count;
       Serial.print("soilhum=");
       Serial.println(soilhum); 
       relay_off();
       TIMSK1=(0<<TOIE1);
       delay(1000);
   }
   if(inter_num>3) inter_num=0;
}
//编码器中断调用函数
void rotaryEncoderChanged(){ // when CLK_PIN is FALLING
  unsigned long temp = millis();
  if(temp - t < 200) // 去抖
    return;
  t = temp;
  
  // DT_PIN的状态代表正转或反转
  count += digitalRead(DT_PIN) == HIGH ? 1 : -1;
}

//按键中断服务函数
void setmode(){
   inter_num++;
}

//定时器中断服务函数
ISR(TIMER1_OVF_vect){
  TCNT1=0xC2F6;//重装初值
  relay_on();//打开继电器
  TIMSK1=(0<<TOIE1); //溢出中断失能
}

